"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.createNodeAsTool = createNodeAsTool;
exports.getNodeAsTool = getNodeAsTool;
const tools_1 = require("@langchain/core/tools");
const zod_1 = require("zod");
function encodeDotNotation(key) {
    return key.replace(/\./g, '__').replace('__value', '');
}
function decodeDotNotation(key) {
    return key.replace(/__/g, '.');
}
function traverseObject(obj, path = [], results = new Map()) {
    for (const [key, value] of Object.entries(obj)) {
        const currentPath = [...path, key];
        const fullPath = currentPath.join('.');
        if (typeof value === 'string' && value.startsWith("{{ '__PLACEHOLDER")) {
            results.set(encodeDotNotation(fullPath), value);
        }
        else if (Array.isArray(value)) {
            traverseArray(value, currentPath, results);
        }
        else if (typeof value === 'object' && value !== null) {
            if (Object.keys(value).length > 0) {
                traverseObject(value, currentPath, results);
            }
        }
    }
    return results;
}
function traverseArray(arr, path, results) {
    arr.forEach((item, index) => {
        const currentPath = [...path, index.toString()];
        const fullPath = currentPath.join('.');
        if (typeof item === 'string' && item.startsWith("{{ '__PLACEHOLDER")) {
            results.set(encodeDotNotation(fullPath), item);
        }
        else if (Array.isArray(item)) {
            traverseArray(item, currentPath, results);
        }
        else if (typeof item === 'object' && item !== null) {
            traverseObject(item, currentPath, results);
        }
    });
}
function buildStructureFromMatches(baseKey, matchingKeys, values) {
    const result = {};
    for (const matchingKey of matchingKeys) {
        const decodedKey = decodeDotNotation(matchingKey);
        const remainingPath = decodedKey
            .slice(baseKey.length)
            .split('.')
            .filter((k) => k !== '');
        let current = result;
        for (let i = 0; i < remainingPath.length - 1; i++) {
            if (!(remainingPath[i] in current)) {
                current[remainingPath[i]] = {};
            }
            current = current[remainingPath[i]];
        }
        const lastKey = remainingPath[remainingPath.length - 1];
        current[lastKey ?? matchingKey] = values[matchingKey];
    }
    return Object.keys(result).length === 0 ? values[encodeDotNotation(baseKey)] : result;
}
function extractPlaceholderDescription(value) {
    const match = value.match(/{{ '__PLACEHOLDER:\s*(.+?)\s*' }}/);
    return match ? match[1] : 'No description provided';
}
function createNodeAsTool(node, ctx, nodeParameters) {
    const placeholderValues = traverseObject(nodeParameters);
    const schemaObj = {};
    for (const [key, value] of placeholderValues.entries()) {
        const description = extractPlaceholderDescription(value);
        schemaObj[key] = zod_1.z.string().describe(description);
    }
    const schema = zod_1.z.object(schemaObj).required();
    const toolDescription = ctx.getNodeParameter('toolDescription', 0, node.description.description);
    const tool = new tools_1.DynamicStructuredTool({
        name: node.description.name,
        description: toolDescription ? toolDescription : node.description.description,
        schema,
        func: async (functionArgs) => {
            const ctxProxy = new Proxy(ctx, {
                get(target, prop, receiver) {
                    if (prop === 'getNodeParameter') {
                        return new Proxy(target.getNodeParameter, {
                            apply(targetMethod, thisArg, argumentsList) {
                                const [key] = argumentsList;
                                if (typeof key !== 'string') {
                                    return Reflect.apply(targetMethod, thisArg, argumentsList);
                                }
                                const encodedKey = encodeDotNotation(key);
                                const matchingKeys = Array.from(placeholderValues.keys()).filter((k) => k.startsWith(encodedKey));
                                if (matchingKeys.length > 0) {
                                    const res = buildStructureFromMatches(encodedKey, matchingKeys, functionArgs);
                                    return res?.[decodeDotNotation(key)] ?? res;
                                }
                                return Reflect.apply(targetMethod, thisArg, argumentsList);
                            },
                        });
                    }
                    return Reflect.get(target, prop, receiver);
                },
            });
            ctxProxy.addInputData("ai_tool", [[{ json: functionArgs }]]);
            const result = await node.execute?.bind(ctxProxy)();
            const mappedResults = result?.[0]?.flatMap((item) => item.json);
            ctxProxy.addOutputData("ai_tool", 0, [
                [{ json: { response: mappedResults } }],
            ]);
            return JSON.stringify(mappedResults);
        },
    });
    return tool;
}
function getNodeAsTool(ctx, node, nodeParameters) {
    return {
        response: createNodeAsTool(node, ctx, nodeParameters),
    };
}
//# sourceMappingURL=CreateNodeAsTool.js.map